﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;
using System.Management;

namespace TweakAPI.Tweak
{
    /// <summary>
    /// Provides possibilities to customize monitor settings.
    /// </summary>
    public static class Monitor
    {
        /// <summary>
        /// Changes the state of the monitor.
        /// </summary>
        public static void SetMonitorStatus(IntPtr handle, Helper.MonitorState ms)
        {
            switch (ms)
            {
                case Helper.MonitorState.On:
                    Win32Api.SendMessage(handle, Convert.ToInt32(Win32Api.Params.WM_SYSCOMMAND), Convert.ToInt32(Win32Api.Params.SC_MONITORPOWER),
                        Convert.ToInt32(Win32Api.Params.TURN_MONITOR_ON));
                    break;
                case Helper.MonitorState.Off:
                    Win32Api.SendMessage(handle, Convert.ToInt32(Win32Api.Params.WM_SYSCOMMAND), Convert.ToInt32(Win32Api.Params.SC_MONITORPOWER),
                        Convert.ToInt32(Win32Api.Params.TURN_MONITOR_OFF));
                    break;
            }
        }

        /// <summary>
        /// Returns the brightness of the monitor, or changes it.
        /// </summary>
        public static int Brightness
        {
            get
            {
                try
                {
                    int currbrightness = 0; int i = 0;
                    ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\WMI", "SELECT * FROM WmiMonitorBrightness");
                    foreach (ManagementObject queryObj in searcher.Get())
                    {
                        currbrightness = int.Parse(queryObj.GetPropertyValue("CurrentBrightness").ToString());
                        i += 1;
                    }
                    return currbrightness;
                }
                catch (Exception ex)
                { throw ex; }
            }
            set
            {
                SetBrightness(value);
            }
        }

        /// <summary>
        /// Method for changing the monitor's brightness.
        /// </summary>
        /// <param name="value"></param>
        private static void SetBrightness(int value)
        {
            ManagementObjectSearcher searcher = new ManagementObjectSearcher(@"root\WMI", "SELECT * FROM WmiMonitorBrightness");
            string instance = "";
            foreach (ManagementObject queryObj in searcher.Get())
            {
                instance = queryObj.GetPropertyValue("InstanceName").ToString();
            }
            ManagementObject classInstance = new ManagementObject(@"root\WMI", @"WmiMonitorBrightnessMethods.InstanceName='" + instance + "'", null);
            ManagementBaseObject inParams = classInstance.GetMethodParameters("WmiSetBrightness");
            inParams["Brightness"] = value;
            inParams["Timeout"] = 5000;
            ManagementBaseObject outParams = classInstance.InvokeMethod("WmiSetBrightness", inParams, null);
        }

        /// <summary>
        /// The default gamma value of most displays.
        /// </summary>
        public const uint DefaultGamma = 10;

        /// <summary>
        /// Changes the gamma value of the monitor.
        /// </summary>
        public static uint Gamma
        {
            set
            {
                Win32Api.RAMP s_ramp = new Win32Api.RAMP();
                s_ramp.Red = new ushort[256];
                s_ramp.Green = new ushort[256];
                s_ramp.Blue = new ushort[256];
                for (int i = 1; i < 256; i++)
                {
                    s_ramp.Red[i] = Win32Api.InlineAssignHelper(ref s_ramp.Green[i], Win32Api.InlineAssignHelper(ref s_ramp.Blue[i], Convert.ToUInt16((Math.Min(65535, Math.Max(0, Math.Pow((i + 1) / 256.0, value * 0.1) * 65535 + 0.5))))));
                }
                Win32Api.SetDeviceGammaRamp(Win32Api.GetDC(IntPtr.Zero), ref s_ramp);
            }
        }

        /// <summary>
        /// Changes the orientation of the monitor.
        /// </summary>
        public static Helper.Orientation Orientation
        {
            get
            {
                Win32Api.DEVMODE dm = new Win32Api.DEVMODE();
                Win32Api.EnumDisplaySettings(null, Win32Api.ENUM_CURRENT_SETTINGS, ref dm);
                return (Helper.Orientation)dm.dmDisplayOrientation;
            }
            set
            {
                Win32Api.DEVMODE dm = new Win32Api.DEVMODE();
                Win32Api.DISP_CHANGE rotRet = Win32Api.DISP_CHANGE.Successful;

                dm.dmDeviceName = new string(Convert.ToChar(0), 32);
                dm.dmFormName = new string(Convert.ToChar(0), 32);
                dm.dmSize = Convert.ToInt16(Marshal.SizeOf(dm));

                if (Win32Api.EnumDisplaySettings(null, Win32Api.ENUM_CURRENT_SETTINGS, ref dm))
                {
                    switch ((Helper.Orientation)dm.dmDisplayOrientation)
                    {
                        case Helper.Orientation.Portrait:
                            Win32Api.SwapExtents(ref dm.dmPelsHeight, ref dm.dmPelsWidth);
                            break;
                        case Helper.Orientation.PortraitReversed :
                            Win32Api.SwapExtents(ref dm.dmPelsHeight, ref dm.dmPelsWidth);
                            break;
                    }

                    dm.dmDisplayOrientation = Convert.ToInt16(value);
                    switch (value)
                    {
                        case Helper.Orientation.Portrait:
                            Win32Api.SwapExtents(ref dm.dmPelsHeight, ref dm.dmPelsWidth);
                            break;
                        case Helper.Orientation.PortraitReversed:
                            Win32Api.SwapExtents(ref dm.dmPelsHeight, ref dm.dmPelsWidth);
                            break;
                    }
                    dm.dmFields = Win32Api.DM_DISPLAYORIENTATION | Win32Api.DM_PELSHEIGHT | Win32Api.DM_PELSWIDTH;
                    rotRet = Win32Api.ChangeDisplaySettingsEx(null, ref dm, IntPtr.Zero, Win32Api.CDS_TEST, IntPtr.Zero);

                    if (rotRet == Win32Api.DISP_CHANGE.Failed)
                    { }
                    else
                    {
                        Win32Api.ChangeDisplaySettingsEx(null, ref dm, IntPtr.Zero, Win32Api.CDS_UPDATEREGISTRY, IntPtr.Zero);
                    }
                }
            }
        }
    }
}
